# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class OneCode(Barcode):
    """
    >>> bc = OneCode()
    >>> bc # doctest: +ELLIPSIS
    <....OneCode object at ...>
    >>> print(bc.render_ps_code('0123456709498765432101234567891')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 213 12
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <30313233343536373039343938373635343332313031323334353637383931>
    <>
    /onecode /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('0123456709498765432101234567891', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'onecode'
    aliases = ('usps onecode', 'uspsonecode', 'usps-onecode', 'usps_onecode')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=0.175, textyoffset=-9, textsize=12)

        @property
        def code_bbox(self):
            height = self.lookup_option('height')
            return [0, 0, 64*(1.44+1.872)+1.44, height*DPI]

        @property
        def text_bbox(self):
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            if self.lookup_option('includetext'):
                return [0, textyoffset, (12-1)*7+8+textsize*0.6, textyoffset+textsize]
            else:
                return self.code_bbox
    renderer = _Renderer

if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Raw(Barcode):
    """
    >>> bc = Raw()
    >>> bc # doctest: +ELLIPSIS
    <....Raw object at ...>
    >>> print(bc.render_ps_code('331132131313411122131311333213114131131221323')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 90 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <33333131333231333133313334313131323231333133313133333332313331313431333
     1313331323231333233>
    <>
    /raw /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('331132131313411122131311333213114131131221323', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'raw'
    aliases = ()

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            textyoffset=-7, textsize=10, height=1)

        def _code_bbox(self, codestring):
            """
            >>> r = Raw._Renderer({})
            >>> r._code_bbox('331132131313411122131311333213114131131221323')
            [0, 0, 90, 72.0]
            """
            height = self.lookup_option('height')
            return [0, 0, sum(int(c) for c in list(codestring)), height*DPI]

        def build_params(self, codestring):
            params = super(Raw._Renderer, self).build_params(codestring)
            params['bbox'] = "%d %d %d %d" % self._boundingbox(
                self._code_bbox(codestring), self._code_bbox(codestring))
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
import re
from .base import Barcode, MatrixCodeRenderer, DPI
from .util import cap_unescape

metrics = (
    # rows cols regh regv rscw rsbl
    [10,  10,    1,    1,    5,  1],
    [12,  12,    1,    1,    7,  1],
    [14,  14,    1,    1,   10,  1],
    [16,  16,    1,    1,   12,  1],
    [18,  18,    1,    1,   14,  1],
    [20,  20,    1,    1,   18,  1],
    [22,  22,    1,    1,   20,  1],
    [24,  24,    1,    1,   24,  1],
    [26,  26,    1,    1,   28,  1],
    [32,  32,    2,    2,   36,  1],
    [36,  36,    2,    2,   42,  1],
    [40,  40,    2,    2,   48,  1],
    [44,  44,    2,    2,   56,  1],
    [48,  48,    2,    2,   68,  1],
    [52,  52,    2,    2,   84,  1],
    [64,  64,    4,    4,  112,  1],
    [72,  72,    4,    4,  144,  1],
    [80,  80,    4,    4,  192,  1],
    [88,  88,    4,    4,  224,  1],
    [96,  96,    4,    4,  272,  1],
    [104, 104,    4,    4,  336,  1],
    [120, 120,    6,    6,  408,  1],
    [132, 132,    6,    6,  496,  1],
    [144, 144,    6,    6,  620,  1],
    [8,  18,    1,    1,    7,  1],
    [8,  32,    1,    2,   11,  1],
    [12,  26,    1,    1,   14,  1],
    [12,  36,    1,    2,   18,  1],
    [16,  36,    1,    2,   24,  1],
    [16,  48,    1,    2,   28,  1],
    )


class DataMatrix(Barcode):
    """
    >>> bc = DataMatrix()
    >>> bc # doctest: +ELLIPSIS
    <....DataMatrix object at ...>
    >>> print(bc.render_ps_code('This is Data Matrix')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 40 40
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <546869732069732044617461204d6174726978>
    <>
    /datamatrix /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('This is Data Matrix') # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('This is ^068ata Matrix') # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('Fixed size', options=dict(rows=48, columns=48)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('Rectangular', options=dict(rows=16, columns=48)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> #_.show()
    >>> bc.render('^098^099^100^142^164^186^101^102^103^104^105',
    ...     options=dict(raw=True)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'datamatrix'
    aliases = ('data matrix', 'data-matrix', 'data_matrix')

    class _Renderer(MatrixCodeRenderer):
        default_options = dict(
            MatrixCodeRenderer.default_options,
            rows=0, columns=0, prefix='', encoding='ascii', raw=False, parse=False,
            parsefnc=False)

        def _code_bbox(self, codestring):
            rows = self.lookup_option('rows')
            columns = self.lookup_option('columns')
            prefix = self.lookup_option('prefix')
            encoding = self.lookup_option('encoding')
            raw = self.lookup_option('raw')
            parse = self.lookup_option('parse')
            parsefnc = self.lookup_option('parse')
            if parse:
                codestring = cap_unescape(codestring)
            premaps = dict(FNC1=[232], PROG=[234], MAC5=[236], MAC6=[237])
            precws = premaps.get(prefix, [])
            prelen = len(precws)
            barlen = len(codestring)
            cw_length = 0
            if raw:
                encoding = 'raw'
            if encoding == 'raw':
                cw_length = barlen/4+prelen
            elif encoding == 'ascii':
                # BY sub(), codestring NO LONGER CARRY ACTUAL MESSAGE.
                if parsefnc:
                    cw_length += len(re.findall(r'(?<!\^)\^FNC1', codestring))
                    codestring = re.sub(r'(?<!\^)\^FNC1', '', codestring)
                    codestring.replace('^^', '^')
                residue = codestring
                while residue:
                    if residue[:2].isdigit():
                        cw_length += 2
                        residue = residue[2:]
                        continue
                    elif residue[0].isdigit():
                        cw_length += 1
                        residue = residue[1:]
                        continue
                    elif ord(residue[0]) < 128:
                        cw_length += 1
                        residue = residue[1:]
                        continue
                    else:
                        cw_length += 2
                        residue = residue[1:]
            elif encoding in ['c40', 'text', 'x12']:
                enc_types = dict(
                    c40=dict(
                        mode=230, eightbits=True,
                        charmap=(
                            '404142434445464748494a4b4c4d4e4f'
                            '505152535455565758595a5b5c5d5e5f'
                            '03808182838485868788898a8b8c8d8e'
                            '0405060708090a0b0c0d8f9091929394'
                            '950e0f101112131415161718191a1b1c'
                            '1d1e1f2021222324252627969798999a'
                            'c0c1c2c3c4c5c6c7c8c9cacbcccdcecf'
                            'd0d1d2d3d4d5d6d7d8d9dadbdcdddedf')),
                    text=dict(
                        mode=239, eightbits=True,
                        charmap=(
                            '404142434445464748494a4b4c4d4e4f'
                            '505152535455565758595a5b5c5d5e5f'
                            '03808182838485868788898a8b8c8d8e'
                            '0405060708090a0b0c0d8f9091929394'
                            '95c1c2c3c4c5c6c7c8c9cacbcccdcecf'
                            'd0d1d2d3d4d5d6d7d8d9da969798999a'
                            'c00e0f101112131415161718191a1b1c'
                            '1d1e1f2021222324252627dbdcdddedf')),
                    x12=dict(
                        mode=238, eightbits=False,
                        parsefnc=False,
                        charmap=(
                            'ffffffffffffffffffffffffff00ffff'
                            'ffffffffffffffffffffffffffffffff'
                            '03ffffffffffffffffff01ffffffffff'
                            '0405060708090a0b0c0dffffffff02ff'
                            'ff0e0f101112131415161718191a1b1c'
                            '1d1e1f2021222324252627ffffffffff'
                            'ffffffffffffffffffffffffffffffff'
                            'ffffffffffffffffffffffffffffffff')))
                enc_props = enc_types.get(encoding)
                parsefnc = enc_props.get('parsefnc', parsefnc)
                eightbits = enc_props.get('eightbits')
                charmap = enc_props.get('charmap')
                if parsefnc:
                    cw_length += 2*len(
                        re.findall(r'(?<!\^)\^FNC1', codestring))
                    codestring = re.sub(r'(?<!\^)\^FNC1', '', codestring)
                    codestring.replace('^^', '^')
                residue = codestring
                while residue:
                    ch, residue = residue[:1], residue[1:]
                    if eightbits and ord(ch) > 127:
                        cw_length += 2
                        continue
                    if 0 < ord(ch) < 128:
                        m_ord = int(charmap[ord(ch)*2:ord(ch)*2+2], 16)
                        if m_ord != 255:
                            if ((m_ord >> 6)-1) != 0:
                                cw_length += 1
                            else:
                                pass
                        else:
                            raise ValueError(
                                'Invalid character in X12 encoding')
                # pad for triples
                if (cw_length % 3):
                    cw_length += (3-cw_length % 3)
                # triples->2bytes
                cw_length = (cw_length/3)*2+prelen+1
            else:
                raise ValueError('invalid encoding (%s).' % encoding)
            ucols = rows
            urows = columns
            for rows, cols, regh, regv, rscw, rsbl in metrics:
                mrows = rows - (2*regh)
                mcols = cols - (2*regv)
                ncws = mrows*mcols//8-rscw
                if cw_length > ncws:
                    continue
                if urows and urows != rows:
                    continue
                if ucols and ucols != cols:
                    continue
                break
            return (0, 0, DPI*cols/72.0*2, DPI*rows/72.0*2)

        def build_params(self, codestring):
            """
            >>> import pprint
            >>> pprint.pprint(DataMatrix._Renderer(()).build_params('abcd'))
            {'bbox': '0 0 24 24',
             'codestring': '<61626364>',
             'codetype': (),
             'options': '<>',
             'xscale': 1.0,
             'yscale': 1.0}
            """
            params = super(DataMatrix._Renderer, self).build_params(codestring)
            params['bbox'] = '%d %d %d %d' % (self._boundingbox(
                self._code_bbox(codestring), self._code_bbox(codestring)))
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
import math
from .base import Barcode, MatrixCodeRenderer, DPI
from .util import cap_unescape


class Pdf417(Barcode):
    """
    >>> bc = Pdf417()
    >>> bc # doctest: +ELLIPSIS
    <....Pdf417 object at ...>
    >>> print(bc.render_ps_code('PDF417')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 103 24
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <504446343137>
    <>
    /pdf417 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('PDF417', options=dict()) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('P^068F417', options=dict(parse=True, columns=2, rows=15), scale=2) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('1234', options=dict(columns=2, rows=10), scale=2) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('A truncated PDF417', options=dict(columns=4, compact=True), scale=2) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('String error correction', options=dict(columns=2, eclevel=5), scale=2) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('^453^178^121^239', options=dict(raw=True, columns=2), scale=2) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()

    """
    codetype = 'pdf417'
    aliases = ('pdf-417', 'pdf_417', 'pdf 417')

    class _Renderer(MatrixCodeRenderer):
        default_options = dict(
            MatrixCodeRenderer.default_options,
            dontdraw=False, compact=False, eclevel=-1, columns=2, rows=0, rowmult=3,
            ccc=False, raw=False, parse=False)

        def _code_bbox(self, codestring):
            compact = self.lookup_option('compact')
            columns = self.lookup_option('columns')
            rowmult = self.lookup_option('rowmult')
            rows = self.lookup_option('rows')
            eclevel = self.lookup_option('eclevel')
            raw = self.lookup_option('raw')
            codestring = cap_unescape(codestring)
            m = 0
            if not raw:
                blen = len(codestring)
                if blen % 6 == 0:
                    m = (blen/6)*5+1
                else:
                    m = ((blen-blen % 6)/6)*5+blen % 6+1
            else:
                m = len(codestring)
            if eclevel == -1:
                if m <= 40:
                    eclevel = 2
                elif 41 <= m <= 160:
                    eclevel = 3
                elif 161 <= m <= 320:
                    eclevel = 4
                else:  # m>=321
                    eclevel = 5
            maxeclevel = int(math.log(928-1-m)/math.log(2))-1
            if eclevel > maxeclevel:
                eclevel = maxeclevel
            k = 2**(eclevel+1)
            if columns == 0:
                columns = int(round(math.sqrt((m+k)/3)))
            if 1 <= columns <= 30:
                c = columns
            else:
                raise ValueError(u'Column out of bound: %d.' % (columns))
            r = int(math.ceil((m+k+1)/(1.0*columns)))
            if r < rows and rows <= 90:
                r = rows
            if r < 3:
                r = 3
            maxeclevel = int(math.log(c*r-1-m)/math.log(2))-1
            if maxeclevel > eclevel:
                eclevel = maxeclevel
                k = 2**(eclevel+1)
            if compact:
                rwid = 17*c+17+17+1
            else:
                rwid = 17*c+17+17+17+17+1
            return (0, 0, DPI*rwid/72.0, DPI*(r/72.0)*rowmult)

        def build_params(self, codestring):
            """
            >>> import pprint
            >>> pprint.pprint(Pdf417._Renderer({}).build_params('abcd'))
            {'bbox': '0 0 103 21',
             'codestring': '<61626364>',
             'codetype': {},
             'options': '<>',
             'xscale': 1.0,
             'yscale': 1.0}
            """
            params = super(Pdf417._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            params['bbox'] = '%d %d %d %d' % self._boundingbox(cbbox, cbbox)
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
import re
from .base import Barcode, LinearCodeRenderer, DPI


CODE128_ESCAPE_RE = re.compile(r'\^\d{3}')
CODE128_CHARS = " !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"


class Code128(Barcode):
    """
    >>> bc = Code128()
    >>> bc # doctest: +ELLIPSIS
    <....Code128 object at ...>
    >>>
    >>> print(bc.render_ps_code('Count0123456789!', options=dict(includetext=True))) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 -7 182 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <436f756e743031323334353637383921>
    <696e636c75646574657874>
    /code128 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('Count0123456789!', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'code128'
    aliases = ('code_128', 'code-128', 'code 128')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=1, includetext=False,
            textyoffset=-7, textsize=10)

        def _count_chars(self, codestring):
            """
            >>> r = Code128._Renderer({})
            >>> r._count_chars('^104^102Count^0990123456789^101!')
            15
            """
            mode = -1
            idx = 0
            count = 0
            while idx < len(codestring):
                if codestring[idx] == '^':
                    code_i = int(codestring[idx+1:idx+4])
                    idx += 4
                else:
                    if mode == 2:
                        code_i = int(codestring[idx:idx+2])
                        idx += 2
                    else:
                        code_i = CODE128_CHARS.find(codestring[idx])
                        idx += 1
                if code_i in (101, 103):
                    mode = 0
                elif code_i in (100, 104):
                    mode = 1
                elif code_i in (99, 105):
                    mode = 2
                count += 1
            return count

        def _code_bbox(self, codestring):
            """
            >>> r = Code128._Renderer({})
            >>> r._code_bbox('^104^102Count^0990123456789^101!')
            [0, 0, 167, 72.0]
            """
            height = self.lookup_option('height')
            return [0, 0, self._count_chars(codestring)*11+2, height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = Code128._Renderer({})
            >>> r._text_bbox('^104^102Count^0990123456789^101!')
            [0, -7, 171.0, 3]
            """
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            textmaxy = textyoffset + textsize
            textmaxx = 11*self._count_chars(codestring)+0.6*textsize
            return [0, textyoffset, textmaxx, textmaxy]

        def build_params(self, codestring):
            params = super(Code128._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params

    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class AusPost(Barcode):
    """
    >>> bc = AusPost()
    >>> bc # doctest: +ELLIPSIS
    <....AusPost object at ...>
    >>> print(bc.render_ps_code('6279438541AaaB 155')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 223 12
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <363237393433383534314161614220313535>
    <>
    /auspost /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('6279438541AaaB 155', options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('6279438541AaaB 155', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('593221132401234567', options=dict(includetext=True, custinfoenc='numeric'), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'auspost'
    aliases = ()

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=0.175,
            includetext=False, textyoffset=-7, textsize=10,
            custinfoenc='character')

        def _code_length(self, codestring):
            head = codestring[:2]
            if head in ['11', '45']:
                codelen = 37
            elif head in ['59']:
                codelen = 52
            elif head in ['62']:
                codelen = 67
            else:
                raise ValueError('Invalid code header.')
            return codelen

        def _code_bbox(self, codestring):
            """
            >>> r = AusPost._Renderer({})
            >>> r._code_bbox('5956439111ABA 9')
            [0, 0, 173.66400000000002, 12.6]
            """
            return [0, 0, self._code_length(codestring)*(1.44+1.872)+1.44,
                    self.lookup_option('height')*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = AusPost._Renderer({})
            >>> r._text_bbox('5956439111ABA 9')
            [0, -7, 173.66400000000002, 3]
            """
            textyoffset = self.lookup_option('textyoffset', 0)
            textsize = self.lookup_option('textsize', 10)
            cminx, cminy, cmaxx, cmaxy = self._code_bbox(codestring)
            return [cminx, textyoffset, cmaxx, textyoffset+textsize]

        def build_params(self, codestring):
            params = super(AusPost._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params
    renderer = _Renderer

if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class RoyalMail(Barcode):
    """
    >>> bc = RoyalMail()
    >>> bc # doctest: +ELLIPSIS
    <....RoyalMail object at ...>
    >>> print(bc.render_ps_code('LE28HS9Z')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 127 12
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <4c4532384853395a>
    <>
    /royalmail /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('LE28HS9Z', options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'royalmail'
    aliases = ('royal mail', 'royal-mail', 'royal_mail', 'rm4scc')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            includetext=False, includecheckintext=False,
            textsize=10, textyoffset=-7, height=0.175)

        def _code_bbox(self, codestring):
            """
            >>> r = RoyalMail._Renderer({})
            >>> r._code_bbox('LE28HS9Z')
            [0, 0, 127.296, 12.6]
            """
            height = self.lookup_option('height')
            codelen = len(codestring)*4+6
            return [0, 0, 1.44+codelen*(1.44+1.872), height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = RoyalMail._Renderer({})
            >>> r._text_bbox('LE28HS9Z')
            [0, 0, 127.296, 12.6]
            """
            if self.lookup_option('includetext'):
                textsize = self.lookup_option('textsize')
                cminx, cminy, cmaxx, cmaxy = self._code_bbox(codestring)
                return [cminx, cminy-textsize, cmaxx, cmaxy]
            else:
                return self._code_bbox(codestring)

        def build_params(self, codestring):
            params = super(RoyalMail._Renderer, self).build_params(codestring)
            params['bbox'] = "%d %d %d %d" % self._boundingbox(
                self._code_bbox(codestring), self._text_bbox(codestring))
            return params
    renderer = _Renderer

if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from os.path import abspath, dirname, join as pathjoin
from binascii import hexlify
from textwrap import TextWrapper
import re

__all__ = ['DEFAULT_PS_CODE_PATH', 'DEFAULT_DISTILL_RE',
           'to_ps', 'cap_unescape', 'dict_to_optstring', 'distill_ps_code',
           'DEFAULT_EPSF_DSC_TEMPLATE', 'DEFAULT_RENDER_COMMAND_TEMPLATE',
           'init_ps_code_template', 'BARCODE_PS_CODE_PATH', 'PS_CODE_TEMPLATE']


# default barcode.ps path and distiller regexp.
DEFAULT_PS_CODE_PATH = pathjoin(
    dirname(abspath(__file__)), 'postscriptbarcode', 'barcode.ps')
DEFAULT_DISTILL_RE = re.compile(r'% --BEGIN TEMPLATE--(.+)% --END TEMPLATE--', re.S)


def _bin(n):
    """
    >>> _bin(0), _bin(1), _bin(63), _bin(4096)
    ('0', '1', '111111', '1000000000000')
    """
    return str(n) if n <= 1 else _bin(n >> 1) + str(n & 1)
try:
    bin
except NameError:
    def bin(n):
        return '0b' + _bin(n)


def zf_bin(n, width):
    """Zero-filled bin() of specified width.

    >>> zf_bin(1, 8)
    '00000001'
    >>> zf_bin(255, 8)
    '11111111'
    >>> zf_bin(256, 8)
    '00000000'
    """
    return bin(n)[2:].zfill(width)[0-width:]


_cap_escape_re = re.compile(r'^\^\d\d\d')


def cap_unescape(msg):
    """
    >>> cap_unescape('This is ^065ztec Code')
    'This is Aztec Code'

    """
    bits = []
    while msg:
        if _cap_escape_re.search(msg):
            oct_ord, msg = msg[1:4], msg[4:]
            bits.append(chr(int(oct_ord, 10) % 256))
        else:
            bits.append(msg[0])
            msg = msg[1:]
    return ''.join(bits)


def to_ps(obj, parlen=False):
    """Converts object into postscript literal

    >>> to_ps(None)
    'null'
    >>> to_ps(123)
    '123'
    >>> to_ps(456.78)
    '456.78'
    >>> to_ps(True), to_ps(False)
    ('true', 'false')
    >>> to_ps('foo bar baz')
    'foo bar baz'
    >>> to_ps('foo bar baz', parlen=True)
    '(foo bar baz)'

    """
    if isinstance(obj, str):
        ret = '%s' % obj
        if parlen:
            ret = '(%s)' % ret
    elif isinstance(obj, bool):
        ret = {True: 'true', False: 'false'}[obj]
    elif isinstance(obj, type(None)):
        ret = 'null'
    else:
        ret = str(obj)
    return ret


def ps_hex_str(s):
    """

    >>> print(ps_hex_str('test testtesttesttest test test testtesttest test \\n'
    ...                  ' sdfojsodfj oij 3240987u098rusipdjf948325u test'))
    <74657374207465737474657374746573747465737420746573742074657374207465737
     474657374746573742074657374200a207364666f6a736f64666a206f696a2033323430
     393837753039387275736970646a66393438333235752074657374>
    """
    try:
        hexlified = hexlify(s)
    except TypeError:
        hexlified = hexlify(s.encode('utf8'))

    if isinstance(hexlified, bytes):
        hexlified = hexlified.decode('ascii')
    wrapped = '<{0}>'.format(hexlified)
    return TextWrapper(subsequent_indent=' ', width=72).fill(wrapped)


def dict_to_optstring(d, none=lambda x: '<>', empty=lambda x: '<>',
                      raw_none=lambda x: '()', raw_empty=lambda x: '()',
                      raw=True):
    """Converts dictionary into ps string in barcode.ps specific format.
    >>> from collections import OrderedDict as odict
    >>> dict_to_optstring(odict([
    ...     ('color','yellow'),
    ...     ('purpose', 'seekagrail'),
    ...     ('spam', True),
    ...     ('egg', False),
    ... ]), raw=True)
    '(color=yellow purpose=seekagrail spam)'
    >>> dict_to_optstring(dict(), raw=True)
    '()'
    >>> dict_to_optstring(None, raw=True)
    '()'
    >>> dict_to_optstring(odict([
    ...     ('color','yellow'),
    ...     ('purpose', 'seekagrail'),
    ...     ('spam', True),
    ...     ('egg', False),
    ... ]), raw=False)
    '<636f6c6f723d79656c6c6f7720707572706f73653d7365656b61677261696c207370616\\n d>'
    >>> dict_to_optstring(dict(), raw=False)
    '<>'
    >>> dict_to_optstring(None, raw=False)
    '<>'
    """
    none_ = none
    empty_ = empty
    if raw:
        none_ = raw_none
        empty_ = raw_empty
    if d is None:
        return none_(d)
    elif d:
        ret = ' '.join(
            (key + {True: '', False: '=%s' % to_ps(value)}[value is True])
            for key, value in list(d.items()) if value is not False)
        if raw:
            return '('+ret+')'
        else:
            return ps_hex_str(ret)
    else:
        return empty_(d)


def distill_ps_code(path_to_ps_code=DEFAULT_PS_CODE_PATH,
                    distill_regexp=DEFAULT_DISTILL_RE):
    """
    Distills barcode procedure code blocks with given path and regexp.

    >>> print(distill_ps_code()) # doctest: +ELLIPSIS
    <BLANKLINE>
    <BLANKLINE>
    %% --BEGIN RESOURCE preamble--
    %%%%BeginResource: Category uk.co.terryburton.bwipp 0.0 0 0 0
    %%%%BeginData:          5 ASCII Lines
    ...
    %%%%EndData
    %%%%EndResource
    %% --END ENCODER hibccodablockf--
    <BLANKLINE>
    <BLANKLINE>
    """
    return distill_regexp.findall(
        open(path_to_ps_code, 'r').read())[0].replace('%', '%%')


DEFAULT_EPSF_DSC_TEMPLATE = """%%!PS-Adobe-2.0
%%%%Pages: (attend)
%%%%Creator: Elaphe powered by barcode.ps
%%%%BoundingBox: %(bbox)s
%%%%LanguageLevel: 2
%%%%EndComments
"""
DEFAULT_RENDER_COMMAND_TEMPLATE = """
gsave
0 0 moveto
%(xscale)f %(yscale)f scale
%(codestring)s
%(options)s
/%(codetype)s /uk.co.terryburton.bwipp findresource exec
grestore
showpage
"""


def init_ps_code_template(epsf_dsc_template=DEFAULT_EPSF_DSC_TEMPLATE,
                          render_command_template=DEFAULT_RENDER_COMMAND_TEMPLATE,
                          ps_code_distiller=distill_ps_code):
    """Initializes postscript code template.
    """
    return '\n'.join([epsf_dsc_template, ps_code_distiller(), render_command_template])


BARCODE_PS_CODE_PATH = distill_ps_code()
PS_CODE_TEMPLATE = init_ps_code_template()


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
import math
from .base import Barcode, MatrixCodeRenderer, DPI
from .util import zf_bin, cap_unescape


AZTEC_CODE_METRICS = [
    # frmt      mlyr icap ncws  bpcw
    ["rune",    0,   0,   0,    6],  # Special metric for rune symbols
    ["compact", 1,   1,   17,   6],
    ["full",    1,   1,   21,   6],
    ["compact", 2,   0,   40,   6],
    ["full",    2,   1,   48,   6],
    ["compact", 3,   0,   51,   8],
    ["full",    3,   1,   60,   8],
    ["compact", 4,   0,   76,   8],
    ["full",    4,   1,   88,   8],
    ["full",    5,   1,   120,  8],
    ["full",    6,   1,   156,  8],
    ["full",    7,   1,   196,  8],
    ["full",    8,   1,   240,  8],
    ["full",    9,   1,   230,  10],
    ["full",    10,  1,   272,  10],
    ["full",    11,  1,   316,  10],
    ["full",    12,  1,   364,  10],
    ["full",    13,  1,   416,  10],
    ["full",    14,  1,   470,  10],
    ["full",    15,  1,   528,  10],
    ["full",    16,  1,   588,  10],
    ["full",    17,  1,   652,  10],
    ["full",    18,  1,   720,  10],
    ["full",    19,  1,   790,  10],
    ["full",    20,  1,   864,  10],
    ["full",    21,  1,   940,  10],
    ["full",    22,  1,   1020, 10],
    ["full",    23,  0,   920,  12],
    ["full",    24,  0,   992,  12],
    ["full",    25,  0,   1066, 12],
    ["full",    26,  0,   1144, 12],
    ["full",    27,  0,   1224, 12],
    ["full",    28,  0,   1306, 12],
    ["full",    29,  0,   1392, 12],
    ["full",    30,  0,   1480, 12],
    ["full",    31,  0,   1570, 12],
    ["full",    32,  0,   1664, 12]]


class AztecCode(Barcode):
    """
    >>> bc = AztecCode()
    >>> bc # doctest: +ELLIPSIS
    <....AztecCode object at ...>
    >>> print(bc.render_ps_code('00100111001000000101001101111000010100111100101000000110', options=dict(raw=True))) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 30 30
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <30303130303131313030313030303030303130313030313130313131313030303031303
     13030313131313030313031303030303030313130>
    <726177>
    /azteccode /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('00100111001000000101001101111000010100111100101000000110', options=dict(raw=True)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('25', options=dict(format='rune')) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('ABC123', options=dict(format='compact')) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('ABC123', options=dict(layers=3, format='full')) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('This is ^065ztec Code', options=dict(parse=True, eclevel=50, ecaddchars=0)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('This is Aztec Code') # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'azteccode'
    aliases = ('aztec code', 'aztec-code', 'aztec_code', 'aztec')

    class _Renderer(MatrixCodeRenderer):
        default_options = dict(
            MatrixCodeRenderer.default_options,
            readerinit=False, layers=-1, eclevel=23, ecaddchars=3,
            format=None, parse=False, raw=False, dontdraw=False)

        def _code_bbox(self, codestring):
            format_ = self.lookup_option('format')
            raw = self.lookup_option('raw')
            parse = self.lookup_option('parse')
            if parse:
                codestring = cap_unescape(codestring)
            msgbits = ''
            if format_ != 'rune':
                if raw:
                    msgbits = codestring
                else:
                    barlen = len(codestring)
                    if barlen < 32:
                        cc = zf_bin(barlen, 5)
                    else:
                        cc = zf_bin(barlen-31, 16)
                    msgbits = '11111'
                    msgbits += cc
                    for ch in codestring:
                        msgbits += zf_bin(ord(ch), 8)
            readerinit = self.lookup_option('readerinit')
            layers = self.lookup_option('layers')
            eclevel = self.lookup_option('eclevel')
            ecaddchars = self.lookup_option('ecaddchars')
            for frmt, mlyr, icap, ncws, bpcw in AZTEC_CODE_METRICS:
                ok = True
                numecw = int(math.ceil(ncws*eclevel/100.0+ecaddchars))
                if len(msgbits) == 0:
                    numecw = 0
                numdcw = ncws-numecw
                if format_ and format_ != frmt:
                    ok = False
                if readerinit and icap != 1:
                    ok = False
                if layers != -1 and layers != mlyr:
                    ok = False
                if math.ceil(len(msgbits)/bpcw) > numdcw:
                    ok = False
                if ok:
                    break
            else:
                raise ValueError(u'No appropreate mode.')
            layers = mlyr
            format_ = frmt
            size = 0

            if format_ == 'full':
                size = ((13+layers*4)+2)+int((layers+10.5)/7.5-1)*2
            else:
                size = ((9+layers*4)+2)
            return (0, 0, DPI*size*2/72.0, DPI*size*2/72.0)

        def build_params(self, codestring):
            """
            >>> import pprint
            >>> pprint.pprint(AztecCode._Renderer({}).build_params('abcd'))
            {'bbox': '0 0 30 30',
             'codestring': '<61626364>',
             'codetype': {},
             'options': '<>',
             'xscale': 1.0,
             'yscale': 1.0}
            """
            params = super(AztecCode._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            params['bbox'] = '%d %d %d %d' % (self._boundingbox(cbbox, cbbox))
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class UpcA(Barcode):
    """
    >>> bc = UpcA()
    >>> bc # doctest: +ELLIPSIS
    <....UpcA object at ...>
    >>> print(bc.render_ps_code('78858101497')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 95 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3738383538313031343937>
    <>
    /upca /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('78858101497', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'upca'
    aliases = ('upc_a', 'upc-a', 'upc a')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            includetext=False, textsize=12, textyoffset=-4, height=1)

        @property
        def code_bbox(self):
            height = self.lookup_option('height')
            return [0, 0, 3+12*7+3+5, DPI*height]

        @property
        def text_bbox(self):
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            textmaxh = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [-7, textyoffset, 96+textsize*0.5, textmaxh]
            else:
                return self.code_bbox
    renderer = _Renderer


class UpcE(Barcode):
    """
    >>> bc = UpcE()
    >>> bc # doctest: +ELLIPSIS
    <....UpcE object at ...>
    >>> print(bc.render_ps_code('0123456')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 53 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <30313233343536>
    <>
    /upce /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('0123456', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'upce'
    aliases = ('upc_e', 'upc-e', 'upc e')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            textsize=12, textyoffset=-4, height=1)

        @property
        def code_bbox(self):
            height = self.lookup_option('height')
            return [0, 0, 4+6*7+7, DPI*height]

        @property
        def text_bbox(self):
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            textmaxh = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [-7, textyoffset, 6*7+11+textsize*0.6, textmaxh]
            else:
                return self.code_bbox
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Plessey(Barcode):
    """
    >>> bc = Plessey()
    >>> bc # doctest: +ELLIPSIS
    <....Plessey object at ...>
    >>> print(bc.render_ps_code('01234ABCD')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 265 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <303132333441424344>
    <>
    /plessey /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('01234ABCD', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('01234ABCD', options=dict(includetext=True, includecheckintext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('01234ABCD', options=dict(unidirectional=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'plessey'
    aliases = ()

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            dontdraw=False, includetext=False, validatecheck=False,
            includecheckintext=False, unidirectional=False,
            textsize=10, textyoffset=-7, height=1)

        def _code_bbox(self, codestring):
            """
            >>> r = Plessey._Renderer({})
            >>> r._code_bbox('A012345ABCDB')
            [0, 0, 325, 72.0]
            """
            height = self.lookup_option('height')
            unidirectional = self.lookup_option('unidirectional')
            endchar = 8 if unidirectional else 25
            return [0, 0, 20+(len(codestring)+2)*20+endchar, height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = Plessey._Renderer({})
            >>> r._text_bbox('A012345ABCDB')
            [0, -7, 325, 3]
            """
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            cminx, cminy, cmaxx, cmaxy = self._code_bbox(codestring)
            return [cminx, textyoffset, cmaxx, textyoffset+textsize]

        def build_params(self, codestring):
            params = super(Plessey._Renderer, self).build_params(codestring)
            text_bbox = (self._text_bbox(codestring) if self.lookup_option('includetext')
                         else [0, 0, 0, 0])
            params['bbox'] = "%d %d %d %d" % self._boundingbox(
                self._code_bbox(codestring), text_bbox)
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Symbol(Barcode):
    """
    >>> bc = Symbol()
    >>> bc # doctest: +ELLIPSIS
    <....Symbol object at ...>
    >>> print(bc.render_ps_code('fima')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 38 45
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <66696d61>
    <>
    /symbol /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('fimd', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'symbol'
    aliases = ('symbols', 'fimsymbols', 'fim symbols', 'fim-symbols', 'fim_symbols')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(LinearCodeRenderer.default_options)
        _widths = dict(
            fima=sum([2.25, 2.25, 2.25, 11.25, 2.25, 11.25, 2.25, 2.25, 2.25]),
            fimb=sum([2.25, 6.75, 2.25, 2.25, 2.25, 6.25, 2.25, 2.25, 2.25, 6.75, 2.25]),
            fimc=sum([2.25, 2.25, 2.25, 6.75, 2.25, 6.75, 2.25, 6.75, 2.25, 2.25, 2.25]),
            fimd=sum([2.25, 2.25, 2.25, 2.25, 2.25, 6.75, 2.25, 6.75, 2.25, 2.25, 2.25, 2.25, 2.25]))

        def _code_bbox(self, codestring):
            """
            >>> r = Symbol._Renderer({})
            >>> r._code_bbox('fima')
            [0, 0, 38.25, 45.0]
            """
            return [0, 0, self._widths[codestring], 0.625*DPI]

        def build_params(self, codestring):
            params = super(Symbol._Renderer, self).build_params(codestring)
            params['bbox'] = "%d %d %d %d" % self._boundingbox(
                self._code_bbox(codestring), self._code_bbox(codestring))
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Code2of5(Barcode):
    """
    >>> bc = Code2of5()
    >>> bc # doctest: +ELLIPSIS
    <....Code2of5 object at ...>
    >>> print(bc.render_ps_code('01234567')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 132 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3031323334353637>
    <>
    /code2of5 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('01234567', options=dict(version='iata', includetext=True, includecheck=True, includecheckintext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'code2of5'
    aliases = ('code_2_of_5', 'code 2of5', 'code_2of5',
               'code 2 of 5', 'code-2of5', 'c2of5', 'c-2of5',
               'code25', 'code 25', 'code_25', 'code-25')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=1,
            includecheck=False, includetext=False,
            includecheckintext=False,
            textyoffset=-7, textsize=10)

        def _code_bbox(self, codestring):
            """
            >>> r = Code2of5._Renderer({})
            >>> r._code_bbox('0123456789')
            [0, 0, 160, 72.0]
            """
            height = self.lookup_option('height')
            if self.lookup_option('includecheck'):
                return [0, 0, 6+(len(codestring)+1)*14+14, height*DPI]
            else:
                return [0, 0, 6+len(codestring)*14+14, height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = Code2of5._Renderer({})
            >>> r._text_bbox('0123456789')
            [0, -7, 160, 3]
            """
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            cminx, cminy, cmaxx, cmaxy = self._code_bbox(codestring)
            return [cminx, textyoffset, cmaxx, textyoffset+textsize]

        def build_params(self, codestring):
            params = super(Code2of5._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
"""elaphe -- A Python binding for Barcode Writer In Pure Postscrpt.
"""
from .base import Barcode

__all__ = ['barcode', 'Barcode']

DEFAULT_PLUGINS = [
    'elaphe.ean', 'elaphe.upc', 'elaphe.code128', 'elaphe.code39',
    'elaphe.code93', 'elaphe.i2of5', 'elaphe.rss', 'elaphe.pharmacode',
    'elaphe.code25', 'elaphe.code11', 'elaphe.codabar', 'elaphe.onecode',
    'elaphe.postnet', 'elaphe.royalmail', 'elaphe.auspost', 'elaphe.kix',
    'elaphe.japanpost', 'elaphe.msi', 'elaphe.plessey', 'elaphe.raw',
    'elaphe.symbol', 'elaphe.pdf417', 'elaphe.datamatrix', 'elaphe.qrcode',
    'elaphe.maxicode', 'elaphe.azteccode']

if __name__ == "__main__":
    DEFAULT_PLUGINS = [s.replace('elaphe.', '') for s in DEFAULT_PLUGINS]


def load_plugins():
    for PL in DEFAULT_PLUGINS:
        try:
            __import__(PL, fromlist=[PL])
        except ImportError as e:
            import sys
            sys.stdout.write(u'Warning: %s\n' % e)
    Barcode.update_codetype_registry()
load_plugins()


def barcode(codetype, codestring, options=None, **kw):
    """

    >>> barcode('nonexistent', '977147396801')
    Traceback (most recent call last):
    ...
    ValueError: No renderer for codetype nonexistent
    >>> barcode('qrcode', 'Hello Barcode Writer In Pure PostScript.',
    ...         options=dict(version=9, eclevel='M'), margin=10, data_mode='8bits') # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('code128', '^104^102Count^0990123456789^101!',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('pdf417', '^453^178^121^239',
    ...         options=dict(columns=2, rows=10), margin=1, scale=2) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('upca', '78858101497',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> barcode('upce', '0123456',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('royalmail', 'LE28HS9Z',
    ...         options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('datamatrix', '^142^164^186',
    ...         options=dict(columns=32, rows=32), margin=1, scale=2.0) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('code11', '0123456789',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('code25', '0123456789',
    ...         options=dict(includetext=False, includecheck=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('code39', 'THIS IS CODE39',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('code93', 'THIS IS CODE93',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('japanpost', '1231FZ13XHS',
    ...         options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('azteccode', '00100111001000000101001101111000010100111100101000000110',
    ...         margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('auspost', '5956439111ABA 9',
    ...         options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('ean13', '977147396801',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('interleaved2of5', '24012345678905',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('raw', '331132131313411122131311333213114131131221323',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('kix', '1231FZ13XHS',
    ...         options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('postnet', '012345',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('pharmacode', '117480',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('plessey', '012345ABCD',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('symbol', 'fimd',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('onecode', '0123456709498765432101234567891',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('maxicode', '^059^042^041^059^040^03001^02996152382802^029840^029001^0291Z00004951^029UPSN^02906X610^029159^0291234567^0291^0471^029^029Y^029634 ALPHA DR^029PITTSBURGH^029PA^030^062^004^063',
    ...         options=dict(mode=2), margin=1, scale=4) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('msi', '0123456789',
    ...         options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('rss14', '24012345678905',
    ...         options=dict(linkage=True, includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    >>> barcode('rationalizedCodabar', 'A0123456789B',
    ...         options=dict(includetext=True, includecheck=True), scale=2, margin=10) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ...>
    >>> # _.show()
    """
    # search for codetype registry
    renderer = Barcode.resolve_codetype(codetype)
    if renderer:
        return renderer().render(codestring, options=options, **kw)
    raise ValueError(u'No renderer for codetype %s' % codetype)


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, MatrixCodeRenderer


class MaxiCode(Barcode):
    """
    >>> bc = MaxiCode()
    >>> bc # doctest: +ELLIPSIS
    <....MaxiCode object at ...>
    >>> print(bc.render_ps_code('This is MaxiCode')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 74 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <54686973206973204d617869436f6465>
    <>
    /maxicode /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('This is MaxiCode') # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('This is Maxi^067ode', options=dict(parse=True)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render(
    ...     '152382802^029840^029001^0291Z00004951^029UPSN^02906X610'
    ...     '^029159^0291234567^0291/1^029^029Y^029634 ALPHA DR^029'
    ...     'PITTSBURGH^029PA^029^004',
    ...     options=dict(mode=2, parse=True)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render(
    ...     'ABC123^029840^029001^0291Z00004951^029UPSN^02906X610'
    ...     '^029159^0291234567^0291/1^029^029Y^029634 ALPHA DR^029'
    ...     'PITTSBURGH^029PA^029^004',
    ...     options=dict(mode=3, parse=True)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render(
    ...     '[)>^03001^02996152382802^029840^029001^0291Z00004951'
    ...     '^029UPSN^02906X610^029159^0291234567^0291/1^029^029Y'
    ...     '^029634 ALPHA DR^029PITTSBURGH^029PA^029^004',
    ...     options=dict(mode=2, parse=True)) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'maxicode'
    aliases = ('maxi-code', 'maxi code', 'maxi_code', 'maxi')

    class _Renderer(MatrixCodeRenderer):
        default_options = dict(
            MatrixCodeRenderer.default_options,
            mode=4, sam=-1)

        @property
        def code_bbox(self):
            col, row = [v*2.4945 for v in (29+0.5*2, 32*0.8661+0.5774*2)]
            return (0, 0, col, row)
    renderer = _Renderer

if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
import math
from .base import Barcode, LinearCodeRenderer


class Phamacode(Barcode):
    """
    >>> bc = Phamacode()
    >>> bc # doctest: +ELLIPSIS
    <....Phamacode object at ...>
    >>>
    >>> print(bc.render_ps_code('117480')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 90 22
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <313137343830>
    <>
    /pharmacode /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('117480', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'pharmacode'
    aliases = ()

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=8*2.835, nwidth=0.5*2.835,
            wwidth=1.5*2.835, swidth=1.0*2.835,
            textyoffset=-7, textsize=10)

        def _code_bbox(self, codestring):
            """
            >>> r = Phamacode._Renderer({})
            >>> r._code_bbox('117480')
            [0, 0, 90.72, 22.68]
            """
            return [0, 0, int(math.log(int(codestring), 2))*2*2.835,
                    self.lookup_option('height')]

        def _text_bbox(self, codestring):
            """
            >>> r = Phamacode._Renderer({})
            >>> r._text_bbox('117480')
            [0, -7, 90.72, 3]
            """
            cminx, cminy, cmaxx, cmaxy = self._code_bbox(codestring)
            textsize = self.lookup_option('textsize')
            textyoffset = self.lookup_option('textyoffset')
            return [cminx, textyoffset, cmaxx, textyoffset+textsize]

        def build_params(self, codestring):
            params = super(Phamacode._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params

    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Ean13(Barcode):
    """
    >>> bc = Ean13()
    >>> bc # doctest: +ELLIPSIS
    <....Ean13 object at ...>
    >>> print(bc.render_ps_code('977147396801')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 95 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <393737313437333936383031>
    <>
    /ean13 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('977147396801', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'ean13'
    aliases = ('ean_13', 'ean-13', 'ean 13', 'jan')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            includetext=False, textsize=12, textyoffset=-4, height=1)
        code_bbox = [0, 0, 13*7+4, DPI]

        @property
        def text_bbox(self):
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            textmaxh = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [-10, textyoffset, (12-1)*7+8+textsize*0.5, textmaxh]
            else:
                return self.code_bbox
    renderer = _Renderer


class ISBN(Barcode):
    """
    >>> bc = ISBN()
    >>> bc # doctest: +ELLIPSIS
    <....ISBN object at ...>
    >>> print(bc.render_ps_code('978-1-56592-479')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 95 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3937382d312d35363539322d343739>
    <>
    /isbn /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('978-1-56592-479', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'isbn'
    aliases = ()
    default_options = dict(textyoffset=-4)

    class _Renderer(LinearCodeRenderer):
        code_bbox = [0, 0, 13*7+4, DPI]

        @property
        def text_bbox(self):
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            textmaxh = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [-10, textyoffset-textsize/2.0, (12-1)*7+8+textsize*0.5, textmaxh]
            else:
                return self.code_bbox

        def build_codestring(self, codestring):
            """
            Allows to accept digit-only notation.
            >>> ISBN._Renderer({}).build_codestring('978 1 56592 479') # '(978-1-56592-479)'
            '<3937382d312d35363539322d343739>'
            """
            cs = "%s%s%s-%s-%s%s%s%s%s-%s%s%s" % tuple(c for c in codestring if c in '0123456789')[:12]
            return super(ISBN._Renderer, self).build_codestring(cs)
    renderer = _Renderer


class Ean8(Barcode):
    """
    >>> bc = Ean8()
    >>> bc # doctest: +ELLIPSIS
    <....Ean8 object at ...>
    >>> print(bc.render_ps_code('01335583')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 60 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3031333335353833>
    <>
    /ean8 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('01335583', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'ean8'
    aliases = ('ean_8', 'ean-8', 'ean 8')

    class _Renderer(LinearCodeRenderer):
        code_bbox = [0, 0, 8*7+4, DPI]
        default_options = dict(
            LinearCodeRenderer.default_options,
            includetext=False, textsize=12, textyoffset=-4, height=1)

        @property
        def text_bbox(self):
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            textmaxh = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [4, textyoffset, 7*7+8+textsize*0.5, textmaxh]
            else:
                return self.code_bbox
    renderer = _Renderer


class Ean5(Barcode):
    """
    >>> bc = Ean5()
    >>> bc # doctest: +ELLIPSIS
    <....Ean5 object at ...>
    >>> print(bc.render_ps_code('90200')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 35 50
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3930323030>
    <>
    /ean5 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('977147396801', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'ean5'
    aliases = ('ean_5', 'ean-5', 'ean 5')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            includetext=False, textsize=12, height=0.7)
        default_options.pop('textyoffset', None)

        @property
        def code_bbox(self):
            height = self.lookup_option('height')
            return [0, 0, 5*7, height*DPI]

        @property
        def text_bbox(self):
            height = self.lookup_option('height')
            textyoffset = self.lookup_option('textyoffset', height*DPI+1)
            textsize = self.lookup_option('textsize')
            textminy = textyoffset
            textmaxy = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [-9+13, textminy, 3*9+13+textsize*0.5, textmaxy]
            else:
                return self.code_bbox
    renderer = _Renderer


class Ean2(Barcode):
    """
    >>> bc = Ean2()
    >>> bc # doctest: +ELLIPSIS
    <....Ean2 object at ...>
    >>> print(bc.render_ps_code('05')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 14 50
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3035>
    <>
    /ean2 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('05', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'ean2'
    aliases = ('ean_2', 'ean-2', 'ean 2')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            includetext=False, textsize=12, height=0.7)
        default_options.pop('textyoffset', None)

        @property
        def code_bbox(self):
            height = self.lookup_option('height')
            return [0, 0, 2*7, height*DPI]

        @property
        def text_bbox(self):
            height = self.lookup_option('height')
            textyoffset = self.lookup_option('textyoffset', height*DPI+1)
            textsize = self.lookup_option('textsize')
            textminy = textyoffset
            textmaxy = textyoffset + textsize
            if self.lookup_option('includetext'):
                return [-9+13, textminy, 13+textsize*0.5, textmaxy]
            else:
                return self.code_bbox
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Rss14(Barcode):
    """
    >>> bc = Rss14()
    >>> bc # doctest: +ELLIPSIS
    <....Rss14 object at ...>
    >>> print(bc.render_ps_code('(01)24012345678905')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 97 33
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <283031293234303132333435363738393035>
    <>
    /databaromni /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('(01)24012345678905', scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    >>> bc.render('(01)24012345678905', options=dict(linkage=True, includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'databaromni'
    aliases = ('rss14', 'rss-14', 'rss_14', 'rss 14')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=33/72.0, textyoffset=-7, linkage=False)
        # SP(1),LG(1),D1(16),LF(15),D2(15),D4(15),R5(16),D3(16),SP(1),RG(1)
        _symbol_length = 1+1+16+15+15+15+16+16+1+1

        def _text_bbox(self, codestring):
            """
            >>> r = Rss14._Renderer({})
            >>> r._text_bbox('24012345678905')
            [0, 0, 97, 33]
            """
            if self.lookup_option('includetext'):
                cminx, cminy, cmaxx, cmaxy = self._code_bbox(codestring)
                textyoffset = self.lookup_option('textyoffset')
                textsize = self.lookup_option('textsize')
                cbox_center = (cminx+cmaxx)/2.0
                textminx = min(cminx, cbox_center-(len(codestring))*textsize*0.6/2.0)
                textmaxx = max(cmaxx, cbox_center+(len(codestring))*textsize*0.6/2.0)
                return [textminx, textyoffset, textmaxx, textyoffset+textsize]
            else:
                return self._code_bbox(codestring)

        def _code_bbox(self, codestring):
            """
            >>> r = Rss14._Renderer({})
            >>> r._code_bbox('24012345678905')
            [0, 0, 97, 33]
            """
            height = self.lookup_option('height')
            return [0, 0, int(self._symbol_length/72.0*DPI), int(height*DPI)]

        def build_params(self, codestring):
            params = super(Rss14._Renderer, self).build_params(codestring)
            params['bbox'] = "%d %d %d %d" % self._boundingbox(
                self._code_bbox(codestring), self._text_bbox(codestring))
            return params

    renderer = _Renderer


class RssLimited(Barcode):
    """
    >>> bc = RssLimited()
    >>> bc # doctest: +ELLIPSIS
    <....RssLimited object at ...>
    >>> print(bc.render_ps_code('(01)15012345678907')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 74 10
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <283031293135303132333435363738393037>
    <>
    /databarlimited /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('(01)15012345678907', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'databarlimited'
    aliases = ('rsslimited', 'rss limited', 'rss_limited', 'rss-limited',
               'rss14limited', 'rss14 limited', 'rss14_limited', 'rss14-limited',)

    class _Renderer(Rss14._Renderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            textyoffset=-7, height=10/72.0)
        # SP(1),LG(1),LD(26),CC(18),RD(26),SP(1),RG(1)
        _symbol_length = 1+1+26+18+26+1+1
    renderer = _Renderer


# EXPERIMENTAL
class RssExpanded(Barcode):
    """
    >>> bc = RssExpanded()
    >>> bc # doctest: +ELLIPSIS
    <....RssExpanded object at ...>
    >>> print(bc.render_ps_code('(01)95012345678903(3103)000123')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 90 34
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <283031293935303132333435363738393033283331303329303030313233>
    <>
    /databarexpanded /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('(01)95012345678903(3103)000123', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show() # Broken!
    """
    codetype = 'databarexpanded'
    aliases = ('rssexpanded', 'rss expanded', 'rss_expanded', 'rss-expanded',
               'rss14expanded', 'rss14 expanded', 'rss14_expanded', 'rss14-expanded',)

    class _Renderer(Rss14._Renderer):
        default_options = dict(
            LinearCodeRenderer.default_options, height=34/72.0)

        def _code_bbox(self, codestring):
            """
            >>> r = RssExpanded._Renderer({})
            >>> r._code_bbox('000000010011001010100001000000010000')
            [0, 0, 105, 34]
            """
            height = self.lookup_option('height')
            return [0, 0, int(DPI*15*(len(codestring)//8+3)/72.0), int(height*DPI)]
        _text_bbox = _code_bbox
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8

from __future__ import print_function
import io
from PIL.EpsImagePlugin import EpsImageFile
from . import util

__all__ = [
    'DPI', 'Renderer', 'LinearCodeRenderer',
    'MatrixCodeRenderer', 'Barcode',
]

DPI = 72.0


def fb_lookup(dic, keys, default):
    """Do dict-lookup for multiple key, returning the first hit.
    """
    for key in keys:
        if key in dic:
            return dic[key]
    else:
        return default


class Renderer(object):
    """Base renderer implementation.
    """
    # The default_options is used as a fallback of renderer options.
    # 'Abstract' subclass such as LinearRenderer may override this as::
    #     default_options = dict(superclass.default_options, **kwargs)
    default_options = dict(
        # input processing
        parse=False,
        parsefnc=False,
        raw=False,
        # symbol dimensions
        height=0,
        width=0,
        )

    def __init__(self, codetype, options=None, **kw):
        self.codetype = codetype
        self.options = options
        self.render_options = kw

    def lookup_option(self, key, default=None):
        return (self.options or {}).get(
            key, getattr(self, 'default_options').get(key, default))

    @property
    def text_bbox(self):
        """
        >>> r = Renderer('ean13')
        >>> r.text_bbox == r.code_bbox
        True
        """
        return self.code_bbox

    @property
    def code_bbox(self):
        """
        >>> r = Renderer('ean13')
        >>> r.text_bbox == [0, 0, DPI, DPI]
        True
        """
        return [0, 0, DPI, DPI]

    @property
    def left_margin(self):
        return fb_lookup(self.render_options, ('left_margin', 'margin'), 0)

    @property
    def right_margin(self):
        return fb_lookup(self.render_options, ('right_margin', 'margin'), 0)

    @property
    def top_margin(self):
        return fb_lookup(self.render_options, ('top_margin', 'margin'), 0)

    @property
    def bottom_margin(self):
        return fb_lookup(self.render_options, ('bottom_margin', 'margin'), 0)

    @property
    def x_scale(self):
        scale = fb_lookup(self.render_options, ('scale',), 1.0)
        if isinstance(scale, tuple):
            scale = scale[0]
        return scale

    @property
    def y_scale(self):
        scale = fb_lookup(self.render_options, ('scale',), 1.0)
        if isinstance(scale, tuple):
            scale = scale[1]
        return scale

    @property
    def boundingbox(self):
        return self._boundingbox(self.code_bbox, self.text_bbox)

    def _boundingbox(self, code_bbox, text_bbox):
        text_lbx, text_lby, text_rtx, text_rty = text_bbox
        code_lbx, code_lby, code_rtx, code_rty = code_bbox
        return (self.x_scale*(min(text_lbx, code_lbx)-self.left_margin),
                self.y_scale*(min(text_lby, code_lby)-self.bottom_margin),
                self.x_scale*(max(text_rtx, code_rtx)+self.right_margin),
                self.y_scale*(max(text_rty, code_rty)+self.top_margin))

    def build_codestring(self, codestring):
        return util.ps_hex_str(codestring)

    def build_options_string(self, options):
        return util.dict_to_optstring(options, raw=False)

    def build_params(self, codestring):
        params = {}
        params['bbox'] = "%d %d %d %d" % self.boundingbox
        params['codestring'] = self.build_codestring(codestring)
        params['options'] = self.build_options_string(self.options)
        params['xscale'], params['yscale'] = self.x_scale, self.y_scale
        params['codetype'] = self.codetype
        return params

    def render_ps_code(self, codestring):
        """
        >>> print(Renderer('foo').render_ps_code('BAR')) # doctest: +ELLIPSIS
        %!PS-Adobe-2.0
        %%Pages: (attend)
        %%Creator: Elaphe powered by barcode.ps
        %%BoundingBox: 0 0 72 72
        %%LanguageLevel: 2
        %%EndComments
        ...
        gsave
        0 0 moveto
        1.000000 1.000000 scale
        <424152>
        <>
        /foo /uk.co.terryburton.bwipp findresource exec
        grestore
        showpage
        <BLANKLINE>
        """
        return util.PS_CODE_TEMPLATE % (self.build_params(codestring))

    def render(self, codestring):
        """
        >>> Renderer('foo').render('977147396801') # doctest: +ELLIPSIS
        <PIL.EpsImagePlugin.EpsImageFile ... at ...>
        """
        ps_code_buf = self.render_ps_code(codestring)
        try:
            f = io.BytesIO(ps_code_buf)
        except TypeError:
            f = io.BytesIO(ps_code_buf.encode('utf8'))
        return EpsImageFile(f)


class LinearCodeRenderer(Renderer):
    default_options = dict(
        Renderer.default_options,
        # check digits
        includecheck=False,
        includecheckintext=False,
        # bar properties
        inkspread=0.15,
        # text properties
        includetext=False,
        textfont='Courier',
        textsize=10,
        textgaps=0,
        # text positioning
        textxalign=None,
        textyalign=None,
        textxoffset=0,
        textyoffset=0,
        # border properties
        showborder=False,
        borderwidth=0.5,
        borderleft=10,
        borderright=10,
        bordertop=1,
        borderbottom=1,
        # symbol colors
        barcolor=None,
        backgroundcolor=None,
        textcolor=None,
        bordercolor=None,
        # EAN/UPC add-on
        addontextfont='Courier',
        addontextsize=10,
        addontextxoffset=0,
        addontextyoffset=0,
        # EAN/UPC guards
        guardwhitespace=False,
        guardwidth=6,
        guardheight=7,
        guardleftpos=0,
        guardrightpos=0,
        guardleftypos=0,
        guardrightypos=0,
        # attic
        barratio=1,
        spaceratio=1,
        )


class MatrixCodeRenderer(Renderer):
    default_options = dict(
        Renderer.default_options,
        # symbol colors
        color=None,
        backgrroundcolor=None,
        )


class Barcode(object):
    """Base class of barcode renderers.

    >>> print(Barcode().render_ps_code('')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 72 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <>
    <>
    / /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    """
    codetype = ''
    aliases = ()
    registry = {}
    renderer = Renderer

    @classmethod
    def update_codetype_registry(cls):
        # update registry
        for subclass in cls.__subclasses__():
            cls.registry.update({subclass.codetype.lower(): subclass})
            if hasattr(subclass, 'aliases'):
                cls.registry.update(
                    dict((alias.lower(), subclass) for alias in subclass.aliases))

    @classmethod
    def resolve_codetype(cls, codetype):
        return cls.registry.get(codetype.lower())

    def get_renderer(self, options=None, **kw):
        return self.renderer(self.codetype, options, **kw)

    def render_ps_code(self, codestring, options=None, **kw):
        renderer = self.get_renderer(options, **kw)
        return renderer.render_ps_code(codestring)

    def render(self, codestring, options=None, **kw):
        renderer = self.get_renderer(options, **kw)
        return renderer.render(codestring)

    # for debug
    def _get_build_params(self, codestring='', options=None, **kw):
        renderer = self.get_renderer(options, **kw)
        return renderer.build_params(codestring)


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# -*- coding: utf-8 -*-
import io
import os

from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        os.path.join(os.path.dirname(__file__), *names),
        encoding=kwargs.get('encoding', 'utf8'),
    ) as fp:
        return fp.read()

readme = open('README.rst').read()
history = open('CHANGES.rst').read().replace('.. :changelog:', '')


setup(
    name="elaphe3",
    version='0.1.1',
    packages=['elaphe'],
    exclude_package_data={
        'elaphe': ['postscriptbarcode']},
    package_data={
        'elaphe': ['postscriptbarcode/barcode.ps',
                   'postscriptbarcode/LICENSE']},
    zip_safe=False,
    install_requires=['pillow'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    author="Yasushi Masuda",
    author_email="whosaysni@gmail.com",
    maintainer='Thomas Grainger',
    maintainer_email='elaphe3@graingert.co.uk',
    description="Generates various barcodes using barcode.ps and pillow.",
    long_description=readme + '\n\n' + history,
    license="New BSD",
    keywords="barcode convert postscript image graphics",
    url="https://github.com/graingert/elaphe",
    test_suite='test.suite',
)



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class JapanPost(Barcode):
    """
    >>> bc = JapanPost()
    >>> bc # doctest: +ELLIPSIS
    <....JapanPost object at ...>
    >>> print(bc.render_ps_code('1231FZ13XHS')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 220 12
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <31323331465a3133584853>
    <>
    /japanpost /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('1231FZ13XHS', options=dict(includetext=False), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'japanpost'
    aliases = ()

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=0.175, includetext=False, includecheckintext=False,
            textyoffset=-7, textsize=10)

        @property
        def code_bbox(self):
            height = self.lookup_option('height')
            return [0, 0, (21*3+3)*(1.44+1.872)+1.44, height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = JapanPost._Renderer({})
            >>> r._text_bbox('LE28HS9Z')
            [0, -7, 220.032, 3]
            """
            textyoffset = self.lookup_option('textyoffset')
            textsize = self.lookup_option('textsize')
            cminx, cminy, cmaxx, cmaxy = self.code_bbox
            return [cminx, textyoffset, cmaxx, textyoffset+textsize]

        def build_params(self, codestring):
            params = super(JapanPost._Renderer, self).build_params(codestring)
            cbbox = self.code_bbox
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params
    renderer = _Renderer

if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
from .base import Barcode, LinearCodeRenderer, DPI


class Interleaved2of5(Barcode):
    """
    >>> bc = Interleaved2of5()
    >>> bc # doctest: +ELLIPSIS
    <....Interleaved2of5 object at ...>
    >>> print(bc.render_ps_code('24012345678905')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 136 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <3234303132333435363738393035>
    <>
    /interleaved2of5 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('24012345678905', options=dict(includetext=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'interleaved2of5'
    aliases = ('interleaved_2_of_5', 'interleaved 2of5', 'interleaved_2of5',
               'interleaved 2 of 5', 'interleaved-2of5', 'i2of5', 'i-2of5')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            includecheck=False, includetext=False, textsize=10, textyoffset=-7, height=1)

        def _code_bbox(self, codestring):
            """
            >>> r = Interleaved2of5._Renderer({})
            >>> r._code_bbox('THIS IS CODE39')
            [0, 0, 136, 72.0]
            """
            height = self.lookup_option('height')
            return [0, 0, len(codestring)*9+4+6, height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = Interleaved2of5._Renderer({})
            >>> r._text_bbox('THIS IS CODE39')
            [0, -7, 136.0, 3]
            """
            textyoffset = self.lookup_option('textyoffset', 0)
            textsize = self.lookup_option('textsize', 10)
            textmaxy = textyoffset + textsize
            textmaxx = 9*len(codestring)+4+0.6*textsize
            return [0, textyoffset, textmaxx, textmaxy]

        def build_params(self, codestring):
            params = super(Interleaved2of5._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params
    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()



# coding: utf-8
from __future__ import print_function
import re
from .base import Barcode, LinearCodeRenderer, DPI


CODE93_ESCAPE_RE = re.compile(r'\^\d{3}')
CODE93_CHARS = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ-. $/+%*"


class Code93(Barcode):
    """
    >>> bc = Code93()
    >>> bc # doctest: +ELLIPSIS
    <....Code93 object at ...>
    >>>
    >>> print(bc.render_ps_code('CODE93')) # doctest: +ELLIPSIS
    %!PS-Adobe-2.0
    %%Pages: (attend)
    %%Creator: Elaphe powered by barcode.ps
    %%BoundingBox: 0 0 73 72
    %%LanguageLevel: 2
    %%EndComments
    ...
    gsave
    0 0 moveto
    1.000000 1.000000 scale
    <434f44453933>
    <>
    /code93 /uk.co.terryburton.bwipp findresource exec
    grestore
    showpage
    <BLANKLINE>
    >>> bc.render('CODE93', options=dict(includetext=True, includecheck=True), scale=2, margin=1) # doctest: +ELLIPSIS
    <PIL.EpsImagePlugin.EpsImageFile ... at ...>
    >>> # _.show()
    """
    codetype = 'code93'
    aliases = ('code_93', 'code-93', 'code 93')

    class _Renderer(LinearCodeRenderer):
        default_options = dict(
            LinearCodeRenderer.default_options,
            height=1,
            includecheck=False, includetext=False,
            textyoffset=-7, textsize=10)

        def _codelen(self, codestring):
            if self.lookup_option('includecheck', False):
                codelen = len(codestring)+3
            else:
                codelen = len(codestring)+2
            return codelen

        def _code_bbox(self, codestring):
            """
            >>> r = Code93._Renderer({})
            >>> r._code_bbox('THIS IS CODE93')
            [0, 0, 145, 72.0]
            """
            height = self.lookup_option('height')
            return [0, 0, self._codelen(codestring)*9+1, height*DPI]

        def _text_bbox(self, codestring):
            """
            >>> r = Code93._Renderer({})
            >>> r._text_bbox('THIS IS CODE93')
            [0, -7, 141.0, 3]
            """
            hidestars = self.lookup_option('hidestars', False)
            textyoffset = self.lookup_option('textyoffset', 0)
            textsize = self.lookup_option('textsize', 10)
            textmaxy = textyoffset + textsize
            textmaxx = 9*(len(codestring)+1)+0.6*textsize
            if hidestars:
                textmaxx = 9, textmaxx-9
            return [0, textyoffset, textmaxx, textmaxy]

        def build_params(self, codestring):
            params = super(Code93._Renderer, self).build_params(codestring)
            cbbox = self._code_bbox(codestring)
            if self.lookup_option('includetext'):
                tbbox = self._text_bbox(codestring)
            else:
                tbbox = cbbox
            params['bbox'] = "%d %d %d %d" % self._boundingbox(cbbox, tbbox)
            return params

    renderer = _Renderer


if __name__ == "__main__":
    from doctest import testmod
    testmod()
